@@ -1,7 +1,7 @@
 /*
  * wee-eval.c - evaluate expressions with references to internal vars
  *
- * Copyright (C) 2012-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2012-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -32,7 +32,7 @@
 #include "weechat.h"
 #include "wee-eval.h"
 #include "wee-calc.h"
-#include "wee-config-file.h"
+#include "wee-config.h"
 #include "wee-hashtable.h"
 #include "wee-hdata.h"
 #include "wee-hook.h"
@@ -413,6 +413,7 @@ eval_string_cut (const char *text, int screen)
     if (!tmp)
         return strdup ("");
 
+    error = NULL;
     number = strtol (tmp, &error, 10);
     if (!error || error[0] || (number < 0))
     {
@@ -453,6 +454,7 @@ eval_string_repeat (const char *text)
     if (!tmp)
         return strdup ("");
 
+    error = NULL;
     number = strtol (tmp, &error, 10);
     if (!error || error[0] || (number < 0))
     {
@@ -538,6 +540,7 @@ eval_string_split (const char *text)
     }
     else
     {
+        error = NULL;
         number = strtol (str_number, &error, 10);
         if (!error || error[0] || (number == 0))
             goto end;
@@ -578,6 +581,7 @@ eval_string_split (const char *text)
             }
             else if (strncmp (list_flags[i], "max_items=", 10) == 0)
             {
+                error = NULL;
                 max_items = strtol (list_flags[i] + 10, &error, 10);
                 if (!error || error[0] || (max_items < 0))
                     goto end;
@@ -686,6 +690,7 @@ eval_string_split_shell (const char *text)
     }
     else
     {
+        error = NULL;
         number = strtol (str_number, &error, 10);
         if (!error || error[0] || (number == 0))
             goto end;
@@ -766,6 +771,7 @@ eval_string_regex_group (const char *text, struct t_eval_context *eval_context)
     }
     else
     {
+        error = NULL;
         number = strtol (text, &error, 10);
         if (!error || error[0])
             number = -1;
@@ -897,6 +903,7 @@ eval_string_base_encode (const char *text)
     if (!base)
         goto end;
 
+    error = NULL;
     number = strtol (base, &error, 10);
     if (!error || error[0])
         goto end;
@@ -946,6 +953,7 @@ eval_string_base_decode (const char *text)
     if (!base)
         goto end;
 
+    error = NULL;
     number = strtol (base, &error, 10);
     if (!error || error[0])
         goto end;
@@ -1087,6 +1095,7 @@ eval_string_random (const char *text)
     tmp = string_strndup (text, pos - text);
     if (!tmp)
         goto error;
+    error = NULL;
     min_number = strtoll (tmp, &error, 10);
     if (!error || error[0])
     {
@@ -1095,6 +1104,7 @@ eval_string_random (const char *text)
     }
     free (tmp);
 
+    error = NULL;
     max_number = strtoll (pos + 1, &error, 10);
     if (!error || error[0])
         goto error;
@@ -1454,13 +1464,122 @@ end:
     return (value) ? value : strdup ("");
 }
 
+/*
+ * Returns text with syntax highlighting (using markers, to be replaced by
+ * colors later).
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+eval_syntax_highlight_add_markers (const char *prefix, const char *text,
+                                   const char *suffix)
+{
+    char **value;
+
+    value = string_dyn_alloc (128);
+    if (!value)
+        return NULL;
+
+    string_dyn_concat (value, EVAL_SYNTAX_HL_INC, -1);
+    string_dyn_concat (value, prefix, -1);
+    if (text)
+        string_dyn_concat (value, text, -1);
+    string_dyn_concat (value, suffix, -1);
+    string_dyn_concat (value, EVAL_SYNTAX_HL_DEC, -1);
+
+    return string_dyn_free (value, 0);
+}
+
+/*
+ * Replaces raw highlight markers with color codes defined in option
+ * weechat.color.eval_syntax_colors.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+eval_syntax_highlight_colorize (const char *value)
+{
+    const char *ptr_value;
+    char **result, *pos;
+    int color;
+
+    if (!value)
+        return NULL;
+
+    result = string_dyn_alloc (128);
+    if (!result)
+        return NULL;
+
+    color = -1;
+    ptr_value = value;
+    while (ptr_value && ptr_value[0])
+    {
+        pos = strstr (ptr_value, EVAL_SYNTAX_HL_MARKER);
+        if (!pos)
+        {
+            string_dyn_concat (result, ptr_value, -1);
+            break;
+        }
+        string_dyn_concat (result, ptr_value, pos - ptr_value);
+        ptr_value = pos + strlen (EVAL_SYNTAX_HL_MARKER);
+        if (config_num_eval_syntax_colors > 0)
+        {
+            if (ptr_value[0] == '+')
+                color++;
+            else if (ptr_value[0] == '-')
+                color--;
+        }
+        ptr_value++;
+        if (config_num_eval_syntax_colors > 0)
+        {
+            string_dyn_concat (
+                result,
+                gui_color_get_custom (
+                    (color >= 0) ?
+                    config_eval_syntax_colors[color % config_num_eval_syntax_colors] :
+                    "reset"),
+                -1);
+        }
+    }
+
+    return string_dyn_free (result, 0);
+}
+
+/*
+ * Adds syntax highlighting in text.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+eval_syntax_highlight (const char *text, struct t_eval_context *eval_context)
+{
+    char *value, *value2;
+
+    eval_context->syntax_highlight++;
+
+    value = eval_replace_vars (text, eval_context);
+    value2 = eval_syntax_highlight_colorize (value);
+    if (value)
+        free (value);
+
+    eval_context->syntax_highlight--;
+
+    return value2;
+}
+
 /*
  * Replaces variables, which can be, by order of priority:
+ *  - the string itself without evaluation but with syntax highlighting
+ *    (format: raw_hl:xxx)
  *  - the string itself without evaluation (format: raw:xxx)
+ *  - a string with syntax highlighting (format: hl:xxx)
  *  - a variable from hashtable "user_vars" or "extra_vars"
  *  - a WeeChat home directory, one of: "weechat_config_dir",
  *    "weechat_data_dir", "weechat_cache_dir", "weechat_runtime_dir"
- *  - a string to evaluate (format: eval:xxx)
+ *  - an evaluated string (format: eval:xxx)
  *  - a condition to evaluate (format: eval_cond:xxx)
  *  - a string with escaped chars (format: esc:xxx or \xxx)
  *  - a string with a range of chars (format: chars:range)
@@ -1506,7 +1625,8 @@ end:
  */
 
 char *
-eval_replace_vars_cb (void *data, const char *text)
+eval_replace_vars_cb (void *data,
+                      const char *prefix, const char *text, const char *suffix)
 {
     struct t_eval_context *eval_context;
     struct t_config_option *ptr_option;
@@ -1521,6 +1641,16 @@ eval_replace_vars_cb (void *data, const char *text)
 
     EVAL_DEBUG_MSG(1, "eval_replace_vars_cb(\"%s\")", text);
 
+    if (eval_context->syntax_highlight)
+        return eval_syntax_highlight_add_markers (prefix, text, suffix);
+
+    /* raw text (no evaluation at all), with syntax highlighting */
+    if (strncmp (text, "raw_hl:", 7) == 0)
+    {
+        value = eval_syntax_highlight (text + 7, eval_context);
+        goto end;
+    }
+
     /* raw text (no evaluation at all) */
     if (strncmp (text, "raw:", 4) == 0)
     {
@@ -1528,6 +1658,13 @@ eval_replace_vars_cb (void *data, const char *text)
         goto end;
     }
 
+    /* syntax highlighting */
+    if (strncmp (text, "hl:", 3) == 0)
+    {
+        value = eval_syntax_highlight (text + 3, eval_context);
+        goto end;
+    }
+
     /* variable in hashtable "user_vars" or "extra_vars" */
     ptr_value = hashtable_get (eval_context->user_vars, text);
     if (ptr_value)
@@ -1881,7 +2018,8 @@ end:
 char *
 eval_replace_vars (const char *expr, struct t_eval_context *eval_context)
 {
-    const char *no_replace_prefix_list[] = { "if:", "raw:", NULL };
+    const char *no_replace_prefix_list_std[] = { "if:", "raw:", "raw_hl:", NULL };
+    const char *no_replace_prefix_list_col[] = { "raw:", "raw_hl:", NULL };
     char *result;
     int debug_id;
 
@@ -1891,13 +2029,16 @@ eval_replace_vars (const char *expr, struct t_eval_context *eval_context)
 
     if (eval_context->recursion_count < EVAL_RECURSION_MAX)
     {
-        result = string_replace_with_callback (expr,
-                                               eval_context->prefix,
-                                               eval_context->suffix,
-                                               no_replace_prefix_list,
-                                               &eval_replace_vars_cb,
-                                               eval_context,
-                                               NULL);
+        result = string_replace_with_callback (
+            expr,
+            eval_context->prefix,
+            eval_context->suffix,
+            (eval_context->syntax_highlight) ? 0 : 1,
+            (eval_context->syntax_highlight) ?
+            no_replace_prefix_list_col : no_replace_prefix_list_std,
+            &eval_replace_vars_cb,
+            eval_context,
+            NULL);
     }
     else
     {
@@ -2532,6 +2673,7 @@ eval_expression (const char *expr, struct t_hashtable *pointers,
     eval_context->regex = NULL;
     eval_context->regex_replacement_index = 1;
     eval_context->recursion_count = 0;
+    eval_context->syntax_highlight = 0;
     eval_context->debug_level = 0;
     eval_context->debug_depth = 0;
     eval_context->debug_id = 0;
@@ -2616,6 +2758,7 @@ eval_expression (const char *expr, struct t_hashtable *pointers,
         ptr_value = hashtable_get (options, "debug");
         if (ptr_value && ptr_value[0])
         {
+            error = NULL;
             number = strtol (ptr_value, &error, 10);
             if (error && !error[0] && (number >= 1))
             {
