@@ -1,7 +1,7 @@
 /*
  * gui-curses-chat.c - chat display functions for Curses GUI
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -35,6 +35,7 @@
 #include "../../core/wee-hook.h"
 #include "../../core/wee-string.h"
 #include "../../core/wee-utf8.h"
+#include "../../core/wee-util.h"
 #include "../../plugins/plugin.h"
 #include "../gui-buffer.h"
 #include "../gui-chat.h"
@@ -607,14 +608,16 @@ gui_chat_display_word (struct t_gui_window *window,
     ptr_data = data;
     while (ptr_data && ptr_data[0])
     {
-        chars_displayed += gui_chat_display_prefix_suffix(window, line,
-                                                          word + (ptr_data - data),
-                                                          pre_lines_displayed,
-                                                          lines_displayed,
-                                                          chars_displayed,
-                                                          simulate,
-                                                          apply_style_inactive,
-                                                          nick_offline);
+        chars_displayed += gui_chat_display_prefix_suffix (
+            window,
+            line,
+            word + (ptr_data - data),
+            pre_lines_displayed,
+            lines_displayed,
+            chars_displayed,
+            simulate,
+            apply_style_inactive,
+            nick_offline);
 
         chars_to_display = gui_chat_strlen_screen (ptr_data);
 
@@ -643,6 +646,8 @@ gui_chat_display_word (struct t_gui_window *window,
                                                               nick_offline);
             }
             ptr_data[pos_saved_char] = saved_char;
+            if (pos_saved_char == 0)
+                break;
             ptr_data += pos_saved_char;
         }
         else
@@ -664,6 +669,8 @@ gui_chat_display_word (struct t_gui_window *window,
                                                               apply_style_inactive,
                                                               nick_offline);
             }
+            if (!ptr_data[0])
+                break;
             ptr_data += strlen (ptr_data);
         }
 
@@ -1505,8 +1512,8 @@ gui_chat_display_line (struct t_gui_window *window, struct t_gui_line *line,
         ptr_data = (message_with_tags) ?
             message_with_tags : line->data->message;
         message_with_search = NULL;
-        if ((window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED)
-            && (window->buffer->text_search_where & GUI_TEXT_SEARCH_IN_MESSAGE)
+        if ((window->buffer->text_search == GUI_BUFFER_SEARCH_LINES)
+            && (window->buffer->text_search_where & GUI_BUFFER_SEARCH_IN_MESSAGE)
             && (!window->buffer->text_search_regex
                 || window->buffer->text_search_regex_compiled))
         {
@@ -1528,13 +1535,16 @@ gui_chat_display_line (struct t_gui_window *window, struct t_gui_line *line,
                 gui_chat_display_new_line (window, num_lines, count,
                                            &lines_displayed, simulate);
                 ptr_data++;
-                gui_chat_display_prefix_suffix(window, line,
-                                               ptr_data,
-                                               pre_lines_displayed, &lines_displayed,
-                                               0,
-                                               simulate,
-                                               CONFIG_BOOLEAN(config_look_color_inactive_message),
-                                               0);
+                gui_chat_display_prefix_suffix (
+                    window,
+                    line,
+                    ptr_data,
+                    pre_lines_displayed,
+                    &lines_displayed,
+                    0,
+                    simulate,
+                    CONFIG_BOOLEAN(config_look_color_inactive_message),
+                    0);
             }
 
             gui_chat_get_word_info (window,
@@ -1695,7 +1705,7 @@ gui_chat_display_line (struct t_gui_window *window, struct t_gui_line *line,
     else
     {
         /* display marker if line is matching user search */
-        if (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED)
+        if (window->buffer->text_search == GUI_BUFFER_SEARCH_LINES)
         {
             if (gui_line_search_text (window->buffer, line))
             {
@@ -1765,8 +1775,8 @@ gui_chat_display_line_y (struct t_gui_window *window, struct t_gui_line *line,
     }
 
     /* emphasize text (if searching text) */
-    if ((window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED)
-        && (window->buffer->text_search_where & GUI_TEXT_SEARCH_IN_MESSAGE)
+    if ((window->buffer->text_search == GUI_BUFFER_SEARCH_LINES)
+        && (window->buffer->text_search_where & GUI_BUFFER_SEARCH_IN_MESSAGE)
         && (!window->buffer->text_search_regex
             || window->buffer->text_search_regex_compiled))
     {
@@ -2094,7 +2104,7 @@ gui_chat_get_bare_line (struct t_gui_line *line)
 {
     char *prefix, *message, str_time[256], *str_line;
     const char *tag_prefix_nick;
-    struct tm *local_time;
+    struct timeval tv;
     int length;
 
     prefix = NULL;
@@ -2116,11 +2126,11 @@ gui_chat_get_bare_line (struct t_gui_line *line)
         && CONFIG_STRING(config_look_bare_display_time_format)
         && CONFIG_STRING(config_look_bare_display_time_format)[0])
     {
-        local_time = localtime (&line->data->date);
-        if (strftime (str_time, sizeof (str_time),
-                      CONFIG_STRING(config_look_bare_display_time_format),
-                      local_time) == 0)
-            str_time[0] = '\0';
+        tv.tv_sec = line->data->date;
+        tv.tv_usec = line->data->date_usec;
+        util_strftimeval (str_time, sizeof (str_time),
+                          CONFIG_STRING(config_look_bare_display_time_format),
+                          &tv);
     }
     tag_prefix_nick = gui_line_search_tag_starting_with (line, "prefix_nick_");
 
