@@ -1,7 +1,7 @@
 /*
  * irc-upgrade.c - save/restore IRC plugin data when upgrading WeeChat
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -663,7 +663,6 @@ irc_upgrade_read_cb (const void *pointer, void *data,
                         memcpy (&(irc_upgrade_current_server->lag_check_time), buf, size);
                     irc_upgrade_current_server->lag_next_check = weechat_infolist_time (infolist, "lag_next_check");
                     irc_upgrade_current_server->lag_last_refresh = weechat_infolist_time (infolist, "lag_last_refresh");
-                    irc_upgrade_current_server->last_user_message = weechat_infolist_time (infolist, "last_user_message");
                     irc_upgrade_current_server->last_away_check = weechat_infolist_time (infolist, "last_away_check");
                     irc_upgrade_current_server->last_data_purge = weechat_infolist_time (infolist, "last_data_purge");
                 }
@@ -970,6 +969,7 @@ irc_upgrade_read_cb (const void *pointer, void *data,
                     {
                         irc_raw_message_add_to_list (
                             weechat_infolist_time (infolist, "date"),
+                            weechat_infolist_integer (infolist, "date_usec"),
                             ptr_server,
                             weechat_infolist_integer (infolist, "flags"),
                             weechat_infolist_string (infolist, "message"));
