@@ -1,7 +1,7 @@
 /*
  * irc.c - IRC (Internet Relay Chat) plugin for WeeChat
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -193,6 +193,9 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
     weechat_plugin = plugin;
 
+    irc_signal_quit_received = 0;
+    irc_signal_upgrade_received = 0;
+
     if (!irc_config_init ())
         return WEECHAT_RC_ERROR;
 
@@ -314,7 +317,10 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
     (void) plugin;
 
     if (irc_hook_timer)
+    {
         weechat_unhook (irc_hook_timer);
+        irc_hook_timer = NULL;
+    }
 
     if (irc_signal_upgrade_received)
     {
