@@ -1,7 +1,7 @@
 /*
  * relay-config.c - relay configuration options (file relay.conf)
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -1270,7 +1270,7 @@ relay_config_init ()
                "documentation of function gnutls_priority_init in gnutls "
                "manual, common strings are: \"PERFORMANCE\", \"NORMAL\", "
                "\"SECURE128\", \"SECURE256\", \"EXPORT\", \"NONE\")"),
-            NULL, 0, 0, "NORMAL:-VERS-SSL3.0", NULL, 0,
+            NULL, 0, 0, "NORMAL", NULL, 0,
             &relay_config_check_network_tls_priorities, NULL, NULL,
             &relay_config_change_network_tls_priorities, NULL, NULL,
             NULL, NULL, NULL);
@@ -1460,6 +1460,7 @@ void
 relay_config_free ()
 {
     weechat_config_free (relay_config_file);
+    relay_config_file = NULL;
 
     if (relay_config_regex_allowed_ips)
     {
