@@ -1,7 +1,7 @@
 /*
  * trigger-command.c - trigger command
  *
- * Copyright (C) 2014-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -607,12 +607,12 @@ trigger_command_trigger (const void *pointer, void *data,
                          struct t_gui_buffer *buffer, int argc,
                          char **argv, char **argv_eol)
 {
-    struct t_trigger *ptr_trigger, *ptr_trigger2;
+    struct t_trigger *ptr_trigger, *ptr_trigger2, *ptr_next_trigger;
     struct t_trigger_regex *regex;
-    char *value, **sargv, **items, *input, str_pos[16];
+    char *value, **sargv, **items, *input, str_pos[16], *name;
     char *arg_arguments, *arg_conditions, *arg_regex, *arg_command;
     char *arg_return_code, *arg_post_action;
-    int rc, i, j, type, count, index_option, enable, sargc, num_items, add_rc;
+    int rc, i, j, type, index_option, enable, sargc, num_items, add_rc;
     int regex_count, regex_rc;
 
     /* make C compiler happy */
@@ -1092,28 +1092,15 @@ trigger_command_trigger (const void *pointer, void *data,
             enable = 0;
         else if (weechat_strcmp (argv[1], "restart") == 0)
             enable = 2;
-        if (weechat_strcmp (argv[2], "-all") == 0)
+        for (i = 2; i < argc; i++)
         {
             for (ptr_trigger = triggers; ptr_trigger;
                  ptr_trigger = ptr_trigger->next_trigger)
             {
-                trigger_command_set_enabled (ptr_trigger, enable, argv[1], 0);
-            }
-        }
-        else
-        {
-            for (i = 2; i < argc; i++)
-            {
-                ptr_trigger = trigger_search (argv[i]);
-                if (ptr_trigger)
-                    trigger_command_set_enabled (ptr_trigger, enable, argv[1],
-                                                 1);
-                else
+                if (weechat_string_match (ptr_trigger->name, argv[i], 1))
                 {
-                    weechat_printf_date_tags (NULL, 0, "no_trigger",
-                                              _("%sTrigger \"%s\" not found"),
-                                              weechat_prefix ("error"),
-                                              argv[i]);
+                    trigger_command_set_enabled (ptr_trigger, enable,
+                                                 argv[1], 1);
                 }
             }
         }
@@ -1125,34 +1112,13 @@ trigger_command_trigger (const void *pointer, void *data,
     {
         if (argc < 3)
             goto error;
-        if (weechat_strcmp (argv[2], "-all") == 0)
+        for (i = 2; i < argc; i++)
         {
-            count = triggers_count;
             ptr_trigger = triggers;
             while (ptr_trigger)
             {
-                ptr_trigger2 = ptr_trigger->next_trigger;
-                if (ptr_trigger->hook_running)
-                {
-                    trigger_command_error_running (ptr_trigger, argv[1]);
-                }
-                else
-                {
-                    trigger_free (ptr_trigger);
-                }
-                ptr_trigger = ptr_trigger2;
-            }
-            count = count - triggers_count;
-            if (count > 0)
-                weechat_printf_date_tags (NULL, 0, "no_trigger",
-                                          _("%d triggers removed"), count);
-        }
-        else
-        {
-            for (i = 2; i < argc; i++)
-            {
-                ptr_trigger = trigger_search (argv[i]);
-                if (ptr_trigger)
+                ptr_next_trigger = ptr_trigger->next_trigger;
+                if (weechat_string_match (ptr_trigger->name, argv[i], 1))
                 {
                     if (ptr_trigger->hook_running)
                     {
@@ -1160,19 +1126,16 @@ trigger_command_trigger (const void *pointer, void *data,
                     }
                     else
                     {
+                        name = strdup (ptr_trigger->name);
                         trigger_free (ptr_trigger);
                         weechat_printf_date_tags (NULL, 0, "no_trigger",
                                                   _("Trigger \"%s\" removed"),
-                                                  argv[i]);
+                                                  name);
+                        if (name)
+                            free (name);
                     }
                 }
-                else
-                {
-                    weechat_printf_date_tags (NULL, 0, "no_trigger",
-                                              _("%sTrigger \"%s\" not found"),
-                                              weechat_prefix ("error"),
-                                              argv[i]);
-                }
+                ptr_trigger = ptr_next_trigger;
             }
         }
         goto end;
@@ -1208,41 +1171,45 @@ trigger_command_trigger (const void *pointer, void *data,
         {
             for (j = 0; trigger_config_default_list[j][0]; j++)
             {
-                if (strcmp (trigger_config_default_list[j][0], argv[i]) == 0)
-                    break;
-            }
-            if (trigger_config_default_list[j][0])
-            {
-                ptr_trigger = trigger_search (argv[i]);
-                if (ptr_trigger && ptr_trigger->hook_running)
+                if (weechat_string_match (trigger_config_default_list[j][0],
+                                          argv[i], 1))
                 {
-                    trigger_command_error_running (ptr_trigger, argv[1]);
+                    ptr_trigger = trigger_search (trigger_config_default_list[j][0]);
+                    if (ptr_trigger && ptr_trigger->hook_running)
+                    {
+                        trigger_command_error_running (
+                            ptr_trigger, trigger_config_default_list[j][0]);
+                    }
+                    else
+                    {
+                        if (ptr_trigger)
+                            trigger_free (ptr_trigger);
+                        ptr_trigger = trigger_new (
+                            trigger_config_default_list[j][0],   /* name */
+                            trigger_config_default_list[j][1],   /* enabled */
+                            trigger_config_default_list[j][2],   /* hook */
+                            trigger_config_default_list[j][3],   /* arguments */
+                            trigger_config_default_list[j][4],   /* conditions */
+                            trigger_config_default_list[j][5],   /* regex */
+                            trigger_config_default_list[j][6],   /* command */
+                            trigger_config_default_list[j][7],   /* return code */
+                            trigger_config_default_list[j][8]);  /* post action */
+                        if (ptr_trigger)
+                        {
+                            weechat_printf_date_tags (
+                                NULL, 0, "no_trigger",
+                                _("Trigger \"%s\" restored"),
+                                ptr_trigger->name);
+                        }
+                        else
+                        {
+                            weechat_printf_date_tags (
+                                NULL, 0, "no_trigger",
+                                _("Failed to restore trigger \"%s\""),
+                                trigger_config_default_list[j][0]);
+                        }
+                    }
                 }
-                else
-                {
-                    if (ptr_trigger)
-                        trigger_free (ptr_trigger);
-                    trigger_new (
-                        trigger_config_default_list[j][0],   /* name */
-                        trigger_config_default_list[j][1],   /* enabled */
-                        trigger_config_default_list[j][2],   /* hook */
-                        trigger_config_default_list[j][3],   /* arguments */
-                        trigger_config_default_list[j][4],   /* conditions */
-                        trigger_config_default_list[j][5],   /* regex */
-                        trigger_config_default_list[j][6],   /* command */
-                        trigger_config_default_list[j][7],   /* return code */
-                        trigger_config_default_list[j][8]);  /* post action */
-                    weechat_printf_date_tags (NULL, 0, "no_trigger",
-                                              _("Trigger \"%s\" restored"),
-                                              argv[i]);
-                }
-            }
-            else
-            {
-                weechat_printf_date_tags (
-                    NULL, 0, "no_trigger",
-                    _("%sDefault trigger \"%s\" not found"),
-                    weechat_prefix ("error"), argv[i]);
             }
         }
         goto end;
@@ -1324,116 +1291,116 @@ trigger_command_init ()
            " || input|output|recreate <name>"
            " || set <name> <option> <value>"
            " || rename|copy <name> <new_name>"
-           " || enable|disable|toggle [<name>|-all [<name>...]]"
-           " || restart <name>|-all [<name>...]"
+           " || enable|disable|toggle [<name>|<mask> [<name>|<mask>...]]"
+           " || restart <name>|<mask> [<name>|<mask>...]"
            " || show <name>"
-           " || del <name>|-all [<name>...]"
-           " || restore <name> [<name>...]"
+           " || del <name>|<mask> [<name>|<mask>...]"
+           " || restore <name>|<mask> [<name>|<mask>...]"
            " || default -yes"
            " || monitor [<filter>]"),
-        N_("       list: list triggers (without argument, this list is displayed)\n"
-           "         -o: send list of triggers enabled to buffer (string in English)\n"
-           "        -ol: send list of triggers enabled to buffer (translated string)\n"
-           "         -i: copy list of triggers enabled in command line (for "
-           "sending to buffer) (string in English)\n"
-           "        -il: copy list of triggers enabled in command line (for "
-           "sending to buffer) (translated string)\n"
-           "   listfull: list triggers with detailed info for each trigger\n"
-           "listdefault: list default triggers\n"
-           "        add: add a trigger\n"
-           "     addoff: add a trigger (disabled)\n"
-           " addreplace: add or replace an existing trigger\n"
-           "       name: name of trigger\n"
-           "       hook: signal, hsignal, modifier, line, print, command, "
-           "command_run, timer, config, focus, info, info_hashtable\n"
-           "  arguments: arguments for the hook, depending on hook (separated "
-           "by semicolons):\n"
-           "             signal: name(s) of signal (required)\n"
-           "             hsignal: name(s) of hsignal (required)\n"
-           "             modifier: name(s) of modifier (required)\n"
-           "             line: buffer type (\"formatted\", \"free\" or \"*\"), "
-           "list of buffer masks, tags\n"
-           "             print: buffer, tags, message, strip colors\n"
-           "             command: command (required), description, arguments, "
-           "description of arguments, completion (all arguments except command "
-           "are evaluated, \"${tg_trigger_name}\" is replaced by the trigger "
-           "name, see /help eval)\n"
-           "             command_run: command(s) (required)\n"
-           "             timer: interval (required), align on second, max calls\n"
-           "             config: name(s) of option (required)\n"
-           "             focus: name(s) of area (required)\n"
-           "             info: name(s) of info (required)\n"
-           "             info_hashtable: name(s) of info (required)\n"
-           " conditions: evaluated conditions for the trigger\n"
-           "      regex: one or more regular expressions to replace strings "
-           "in variables\n"
-           "    command: command to execute (many commands can be separated by "
-           "\";\")\n"
-           "return_code: return code in callback (ok (default), ok_eat, error)\n"
-           "post_action: action to take after execution (none (default), "
-           "disable, delete)\n"
-           "   addinput: set input with default arguments to create a trigger\n"
-           "      input: set input with the command used to create the trigger\n"
-           "     output: send the command to create the trigger on the buffer\n"
-           "   recreate: same as \"input\", with option \"addreplace\" instead "
-           "of \"add\"\n"
-           "        set: set an option in a trigger\n"
-           "     option: name of option: name, hook, arguments, conditions, "
-           "regex, command, return_code\n"
-           "             (for help on option, you can type: /help "
-           "trigger.trigger.<name>.<option>)\n"
-           "      value: new value for the option\n"
-           "     rename: rename a trigger\n"
-           "       copy: copy a trigger\n"
-           "     enable: enable trigger(s) (without arguments: enable triggers "
-           "globally)\n"
-           "    disable: disable trigger(s) (without arguments: disable triggers "
-           "globally)\n"
-           "     toggle: toggle trigger(s) (without arguments: toggle triggers "
-           "globally)\n"
-           "    restart: restart trigger(s) (recreate the hooks)\n"
-           "       show: show detailed info on a trigger (with some stats)\n"
-           "        del: delete a trigger\n"
-           "       -all: do action on all triggers\n"
-           "    restore: restore trigger(s) with the default values (works "
-           "only for default triggers)\n"
-           "    default: delete all triggers and restore default ones\n"
-           "    monitor: open the trigger monitor buffer, with optional filter:\n"
-           "     filter: filter hooks/triggers to display (a hook must start "
-           "with \"@\", for example \"@signal\"), many filters can be separated "
-           "by commas; wildcard \"*\" is allowed in each trigger name\n"
-           "\n"
-           "When a trigger callback is called, following actions are performed, "
-           "in this order:\n"
-           "  1. check conditions; if false, exit\n"
-           "  2. replace text using POSIX extended regular expression(s) (if "
-           "defined in trigger)\n"
-           "  3. execute command(s) (if defined in trigger)\n"
-           "  4. exit with a return code (except for modifier, line, focus, "
-           "info and info_hashtable)\n"
-           "  5. perform post action\n"
-           "\n"
-           "Examples (you can also look at default triggers with /trigger "
-           "listdefault):\n"
-           "  add text attributes *bold*, _underline_ and /italic/ (only in "
-           "user messages):\n"
-           "    /trigger add effects modifier weechat_print \"${tg_tag_nick}\" "
-           "\"==\\*([^ ]+)\\*==*${color:bold}${re:1}${color:-bold}*== "
-           "==_([^ ]+)_==_${color:underline}${re:1}${color:-underline}_== "
-           "==/([^ ]+)/==/${color:italic}${re:1}${color:-italic}/\"\n"
-           "  hide nicklist bar on small terminals:\n"
-           "    /trigger add resize_small signal signal_sigwinch "
-           "\"${info:term_width} < 100\" \"\" \"/bar hide nicklist\"\n"
-           "    /trigger add resize_big signal signal_sigwinch "
-           "\"${info:term_width} >= 100\" \"\" \"/bar show nicklist\"\n"
-           "  silently save config each hour:\n"
-           "    /trigger add cfgsave timer 3600000;0;0 \"\" \"\" \"/mute /save\"\n"
-           "  silently save WeeChat session at midnight (see /help upgrade):\n"
-           "    /trigger add session_save signal day_changed \"\" \"\" "
-           "\"/mute /upgrade -save\"\n"
-           "  open trigger monitor and show only modifiers and triggers whose "
-           "name starts with \"resize\":\n"
-           "    /trigger monitor @modifier,resize*"),
+        WEECHAT_CMD_ARGS_DESC(
+            N_("raw[list]: list triggers (without argument, this list is displayed)"),
+            N_("raw[-o]: send list of triggers enabled to buffer (string in English)"),
+            N_("raw[-ol]: send list of triggers enabled to buffer (translated string)"),
+            N_("raw[-i]: copy list of triggers enabled in command line (for "
+               "sending to buffer) (string in English)"),
+            N_("raw[-il]: copy list of triggers enabled in command line (for "
+               "sending to buffer) (translated string)"),
+            N_("raw[listfull]: list triggers with detailed info for each trigger"),
+            N_("raw[listdefault]: list default triggers"),
+            N_("raw[add]: add a trigger"),
+            N_("raw[addoff]: add a trigger (disabled)"),
+            N_("raw[addreplace]: add or replace an existing trigger"),
+            N_("name: name of trigger"),
+            N_("hook: signal, hsignal, modifier, line, print, command, "
+               "command_run, timer, config, focus, info, info_hashtable"),
+            N_("arguments: arguments for the hook, depending on hook (separated "
+               "by semicolons):"),
+            N_("> type `signal`: name(s) of signal (required)"),
+            N_("> type `hsignal`: name(s) of hsignal (required)"),
+            N_("> type `modifier`: name(s) of modifier (required)"),
+            N_("> type `line`: buffer type (\"formatted\", \"free\" or \"*\"), "
+               "list of buffer masks, tags"),
+            N_("> type `print`: buffer, tags, message, strip colors"),
+            N_("> type `command`: command (required), description, arguments, "
+               "description of arguments, completion (all arguments except command "
+               "are evaluated, \"${tg_trigger_name}\" is replaced by the trigger "
+               "name, see /help eval)"),
+            N_("> type `command_run`: command(s) (required)"),
+            N_("> type `timer`: interval (required), align on second, max calls"),
+            N_("> type `config`: name(s) of option (required)"),
+            N_("> type `focus`: name(s) of area (required)"),
+            N_("> type `info`: name(s) of info (required)"),
+            N_("> type `info_hashtable`: name(s) of info (required)"),
+            N_("conditions: evaluated conditions for the trigger"),
+            N_("regex: one or more regular expressions to replace strings "
+               "in variables"),
+            N_("command: command to execute (many commands can be separated by "
+               "\";\")"),
+            N_("return_code: return code in callback (ok (default), ok_eat, error)"),
+            N_("post_action: action to take after execution (none (default), "
+               "disable, delete)"),
+            N_("raw[addinput]: set input with default arguments to create a trigger"),
+            N_("raw[input]: set input with the command used to create the trigger"),
+            N_("raw[output]: send the command to create the trigger on the buffer"),
+            N_("raw[recreate]: same as \"input\", with option \"addreplace\" instead "
+               "of \"add\""),
+            N_("raw[set]: set an option in a trigger"),
+            N_("option: name of option: name, hook, arguments, conditions, "
+               "regex, command, return_code (for help on option, you can type: "
+               "/help trigger.trigger.<name>.<option>)"),
+            N_("value: new value for the option"),
+            N_("raw[rename]: rename a trigger"),
+            N_("raw[copy]: copy a trigger"),
+            N_("raw[enable]: enable triggers "
+               "(without arguments: enable triggers globally)"),
+            N_("raw[disable]: disable triggers "
+               "(without arguments: disable triggers globally)"),
+            N_("raw[toggle]: toggle triggers "
+               "(without arguments: toggle triggers globally)"),
+            N_("raw[restart]: restart triggers (recreate the hooks)"),
+            N_("raw[show]: show detailed info on a trigger (with some stats)"),
+            N_("raw[del]: delete triggers"),
+            N_("raw[restore]: restore triggers with the default values (works "
+               "only for default triggers)"),
+            N_("mask: name where wildcard \"*\" is allowed"),
+            N_("raw[default]: delete all triggers and restore default ones"),
+            N_("raw[monitor]: open the trigger monitor buffer, with optional filter"),
+            N_("filter: filter hooks/triggers to display (a hook must start "
+               "with \"@\", for example \"@signal\"), many filters can be separated "
+               "by commas; wildcard \"*\" is allowed in each trigger name"),
+            "",
+            N_("When a trigger callback is called, following actions are performed, "
+               "in this order:"),
+            N_("  1. check conditions; if false, exit"),
+            N_("  2. replace text using POSIX extended regular expression(s) (if "
+               "defined in trigger)"),
+            N_("  3. execute command(s) (if defined in trigger)"),
+            N_("  4. exit with a return code (except for modifier, line, focus, "
+               "info and info_hashtable)"),
+            N_("  5. perform post action"),
+            "",
+            N_("Examples (you can also look at default triggers with /trigger "
+               "listdefault):"),
+            N_("  add text attributes *bold*, _underline_ and /italic/ (only in "
+               "user messages):"),
+            AI("    /trigger add effects modifier weechat_print \"${tg_tag_nick}\" "
+               "\"==\\*([^ ]+)\\*==*${color:bold}${re:1}${color:-bold}*== "
+               "==_([^ ]+)_==_${color:underline}${re:1}${color:-underline}_== "
+               "==/([^ ]+)/==/${color:italic}${re:1}${color:-italic}/\""),
+            N_("  hide nicklist bar on small terminals:"),
+            AI("    /trigger add resize_small signal signal_sigwinch "
+               "\"${info:term_width} < 100\" \"\" \"/bar hide nicklist\""),
+            AI("    /trigger add resize_big signal signal_sigwinch "
+               "\"${info:term_width} >= 100\" \"\" \"/bar show nicklist\""),
+            N_("  silently save config each hour:"),
+            AI("    /trigger add cfgsave timer 3600000;0;0 \"\" \"\" \"/mute /save\""),
+            N_("  silently save WeeChat session at midnight (see /help upgrade):"),
+            AI("    /trigger add session_save signal day_changed \"\" \"\" "
+               "\"/mute /upgrade -save\""),
+            N_("  open trigger monitor and show only modifiers and triggers whose "
+               "name starts with \"resize\":"),
+            AI("    /trigger monitor @modifier,resize*")),
         "list -i|-il|-o|-ol"
         " || listfull"
         " || listdefault"
@@ -1442,9 +1409,9 @@ trigger_command_init ()
         " || input|output|recreate %(trigger_names)"
         " || set %(trigger_names) %(trigger_options)|name %(trigger_option_value)"
         " || rename|copy %(trigger_names) %(trigger_names)"
-        " || enable %(trigger_names_disabled)|-all %(trigger_names_disabled)|%*"
-        " || disable %(trigger_names_enabled)|-all %(trigger_names_enabled)|%*"
-        " || toggle|restart|del %(trigger_names)|-all %(trigger_names)|%*"
+        " || enable %(trigger_names_disabled)|%*"
+        " || disable %(trigger_names_enabled)|%*"
+        " || toggle|restart|del %(trigger_names)|%*"
         " || show %(trigger_names)"
         " || restore %(trigger_names_default)|%*"
         " || default"
